-----------------------------------------------------------------------------------------------
-- Client Lua Script for DaemonSense
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- DaemonSense Module Definition
-----------------------------------------------------------------------------------------------
local DaemonSense = {} 
local sin = math.sin
local cos =  math.cos
local rad = math.rad
local tSettings = {}
	tSettings.Options = {
		color		= ApolloColor.new("white"),
		size        = 6.5,
	}
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function DaemonSense:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function DaemonSense:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- DaemonSense OnLoad
-----------------------------------------------------------------------------------------------
function DaemonSense:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("DaemonSense.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- DaemonSense OnDocLoaded
-----------------------------------------------------------------------------------------------
function DaemonSense:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "DaemonSenseForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self.Overlay = Apollo.LoadForm(self.xmlDoc, "dsform",'InWorldHudStratum', self)
		if self.Overlay == nil then
			Apollo.AddAddonErrorText(self, "Could not load the  window for some reason.")
			return
		end
		self.Overlay:Show(true, true)
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("dsaids", "OnDaemonSenseOn", self)
		Apollo.RegisterSlashCommand("aidson", "OnDaemonSenseTimerOn", self)
		Apollo.RegisterSlashCommand("aidsoff", "OnDaemonSenseTimerOff", self)
		Apollo.RegisterSlashCommand("aidstest", "OnDaemonSenseTestShout", self)
		Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
		
		
		self.BuffWatchTimer   = ApolloTimer.Create(0.01, true, "onBuffId", self)
		self.TestDisplayTimer = ApolloTimer.Create(5, false, "OnDaemonSenseTestTimerFire", self)
		self.TestDisplayTimer:Stop()
		self.BuffWatchTimer:Stop()

		-- Do additional Addon initialization here
		
		self.tSettings = {}
		self.tSettings.bRunning = false
		self.tSettings.Options = {}
		self.tSettings.Options["color"] = ApolloColor.new("white")
		self.tSettings.Options["size"] = 6.5
		self.tSettings.Options["opacity"] = 1.0
		self.tSettings.Master = Apollo.GetAddonInfo("DaemonSense").strAuthor
		
	end
end

-----------------------------------------------------------------------------------------------
-- DaemonSense Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/dsaids"
function DaemonSense:OnDaemonSenseOn()
	self.wndMain:Invoke() -- show the window
end

function DaemonSense:OnDaemonSenseTimerOn()
self.tSettings.bRunning = true
self.BuffWatchTimer:Start()
ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System,"DaemonSense is now ENABLED!" )
--self:DrawIt()
end

function DaemonSense:OnDaemonSenseTimerOff()
self.tSettings.bRunning = false
ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System,"DaemonSense is now DISDABLED!" )
self.BuffWatchTimer:Stop()
self.Overlay:DestroyAllPixies()

end

function DaemonSense:OnDaemonSenseTestShout()
local iAids = math.random(1,100)
	if iAids >= 90 then 
		ChatSystemLib.Command('/e ' ..  " has got it worse than Freddie Mercury.")
		self:ShoutAids(strSpellId)
		self.BuffWatchTimer:Start()
		self.TestDisplayTimer:Start()
	elseif iAids < 90 and iAids <= 25 then
		ChatSystemLib.Command('/e ' .. " needs to wear protection more often.")
		self:ShoutAids(strSpellId)
		self.BuffWatchTimer:Start()
		self.TestDisplayTimer:Start()
	else
		ChatSystemLib.Command('/e ' ..  " is clean as a whistle.")
		self:ShoutAids(strSpellId) 
		self.BuffWatchTimer:Start()
		self.TestDisplayTimer:Start()

	end
end

function DaemonSense:OnDaemonSenseTestTimerFire()
	self.TestDisplayTimer:Stop()
	self.Overlay:DestroyAllPixies()
	self.BuffWatchTimer:Stop()
end

function DaemonSense:DrawIt()
self.Overlay:DestroyAllPixies()
if GameLib.GetPlayerUnit() ~= nil then
	local uMyself = GameLib.GetPlayerUnit()
	local uMyselfInfo = uMyself:GetInfo()
	self:DrawCircleRemix(uMyself,uMyselfInfo)
else 
	self.Overlay:DestroyAllPixies()
	self.BuffWatchTimer:Stop()
end
--Print("we got to the drawIt")
end
	
function DaemonSense:onBuffId()
local tBuffs = GameLib.GetPlayerUnit():GetBuffs()
	if tBuffs["arHarmful"] then
		local tBuffsHarm = tBuffs["arHarmful"]
		local tBuffsBene = tBuffs["arBeneficial"]
		for key,value in ipairs(tBuffsHarm) do
			local strSpellID = value.idBuff
			local strSpellName = value["splEffect"]:GetName()
			if strSpellName == "Purge" then
				self:ShoutAids(strSpellID)
			end
		end
	end
end
self:DrawIt()
end
		
function DaemonSense:DrawCircleRemix(unit, unitInfo)
-- Print("we got to the drawIt remix")
	local uface = unit:GetFacing()
	local uangle = math.atan2(uface.x, uface.z)
	--Print(tostring(uangle))
	local originPosition= unit:GetPosition()
		
	local originPositionVector= Vector3.New(originPosition.x,originPosition.y,originPosition.z)
--	Print(tostring(originPositionVector))

	self:PlotCircle(originPositionVector,self.tSettings.Options["size"], tSettings.Options["color"],uangle)
end


function DaemonSense:PlotCircle(originPositionVector, dist, color,uangle)
-- Print("even polotting")
	local firstVector= Vector3.New(originPositionVector.x+dist*sin(uangle), originPositionVector.y, originPositionVector.z+dist*cos(uangle))
	local previousVector=firstVector
	local startPoint = GameLib.WorldLocToScreenPoint(previousVector)
	local pPoints = {0,0,0,0}
	for i = 1, 16 do 
		local rotation_point = uangle + rad(-360 / (16/(i)))

		local currentVector = Vector3.New(originPositionVector.x+dist*sin(rotation_point), originPositionVector.y, originPositionVector.z+dist*cos(rotation_point))
		local endPoint = GameLib.WorldLocToScreenPoint(currentVector)

		self.Overlay:AddPixie( { bLine = true, fWidth = 4,  cr = tSettings.Options["color"], loc = { fPoints = pPoints, nOffsets = { startPoint.x, startPoint.y, endPoint.x, endPoint.y } }    } )
		previousVector=currentVector
		startPoint = endPoint
	end
end


-----------------------------------------------------------------------------------------------
-- DaemonSenseForm Functions
-----------------------------------------------------------------------------------------------


function DaemonSense:ShoutAids(strSpellId)
	Sound.PlayFile("aids.wav")
end

function DaemonSense:OnChatMessage(channelCurrent, tMessage)
	if GameLib.GetPlayerUnit() ~= nil then
		if channelCurrent:GetName() == GameLib.GetPlayerUnit():GetGuildName() and tMessage.strSender == self.tSettings.Master then
			local strMessage = ""
				for idx, tSegment in ipairs(tMessage.arMessageSegments) do
					strMessage = strMessage .. tSegment.strText
				end
			if strMessage == "hypecheck!" then
				if self.tSettings.bRunning then
					Print("Hype Check Passed, Stay focused!")
					ChatSystemLib.Command("/w Tsulong Pergo Enabled and Running!")
				elseif  not self.tSettings.bRunning then
					Print("Type /aidson to turn on your circle please. Stay focused!") 
					ChatSystemLib.Command("/w Tsulong Pergo I do not have DaemonSense Running.")
				end
			end	
				--Deal with Whisper Changes on Colors / size
		end
		if channelCurrent:GetName() == "Whisper"  and tMessage.strSender == self.tSettings.Master then
			local strMessage2 = ""
				for idx, tSegment in ipairs(tMessage.arMessageSegments) do
					strMessage2 = strMessage2 .. tSegment.strText
				end
				if strMessage2 == "color gray" then
					Print("Color Force Changed to gray by Tsulong")
					self:Colorize("gray")
				elseif strMessage2 == "color blue" then
					Print("Color Force Changed to blue by Tsulong")
					self:Colorize("blue")
				elseif strMessage2 == "color red" then
					Print("Color Force Changed to red by Tsulong")
					self:Colorize("red")
				elseif strMessage2 == "color green" then
					Print("Color Force Changed to Green by Tsulong")
					self:Colorize("green")
				end
		end	
	end
end

function DaemonSense:Colorize(strColor)
	tSettings.Options["color"] = ApolloColor.new(strColor)
end

--SendVarToRover( "chancur", channelCurrent)
--SedVarToRover("tmsg",tMessage


-----------------------------------------------------------------------------------------------
-- DaemonSense Instance
-----------------------------------------------------------------------------------------------
local DaemonSenseInst = DaemonSense:new()
DaemonSenseInst:Init()
